# Michael Wollensack METAS - 07.06.2016

import clr
clr.AddReference('System.Windows.Forms')
clr.AddReference('Metas.Instr.Driver')
clr.AddReference('Metas.Vna.Database')
clr.AddReference('Metas.Vna.Journal')
clr.AddReference('Metas.Vna.Tools')
from System import Array
from System import DateTime
from System.IO import Directory
from System.IO import DirectoryInfo
from System.IO import File
from System.IO import FileInfo
from System.IO import SearchOption
from System.Collections.Generic import List
from System.Threading import Thread
from System.Windows.Forms import MessageBox
from Metas.Instr.Driver.Vna import VnaSettings
from Metas.Instr.Driver.Vna import VnaSweepMode
from Metas.Instr.Driver.Vna import VnaSegmentTableItem
from Metas.Instr.Driver.Vna import VnaSourceSettings
from Metas.Instr.Driver.Vna import VnaPortSettings
from Metas.Vna.Journal import MeasurementJournalItem
from Metas.Vna.Journal import VnaSettingsJournalItem
from Metas.Vna.Journal import CustomCableSettingsJournalItem
from Metas.Vna.Journal import CustomConnectorSettingsJournalItem
from Metas.Vna.Database.Connectors import Connector
from Metas.Vna.Database.Connectors import ConnectorElectricalSpec
from Metas.Vna.Database.Cables import Cable
from Metas.Vna.Database.Cables import CableElectricalSpec
from Metas.Vna.Tools import Script

s = Script(RootPath)

datadir = RootPath + '\\_OrginalData'
projectdir = RootPath
journalfile = projectdir + '\\Journal_01.vnalog'
measdir = projectdir + '\\Measurements_01'
vna = 'Test_VNA'
cable = 'Custom'
connector = 'Custom'
dutunc = 'DUT_Unc_0.000_0.020'

def AddVnaSettingsToJournal(timestamp, journal):
    start = 70e3
    stop = 70e9
    ifbw = 30
    power = -10
    vnaitem = VnaSettingsJournalItem()
    vnaitem.TimeStamp = timestamp
    vnaitem.VnaDevice = vna
    vnaitem.Settings = VnaSettings(
        SweepMode = VnaSweepMode.SegmentSweep,
        SweepTime = 30,
        DwellTime = 0,
        IFAverageFactor = 1,
        StartFrequency = start,
        StopFrequency = stop,
        Points = 730,
        IFBandwidth = ifbw,
        CenterFrequency = (stop - start) / 2,
        SpanFrequency = stop - start,
        CWFrequency = (stop - start) / 2,
        SegmentTable = Array[VnaSegmentTableItem]([
            VnaSegmentTableItem(StartFrequency = 70e3, StopFrequency = 90e3, StepFrequency = 10e3, IFBandwidth = ifbw),
            VnaSegmentTableItem(StartFrequency = 100e3, StopFrequency = 900e3, StepFrequency = 100e3, IFBandwidth = ifbw),
            VnaSegmentTableItem(StartFrequency = 1e6, StopFrequency = 9e6, StepFrequency = 1e6, IFBandwidth = ifbw),
            VnaSegmentTableItem(StartFrequency = 10e6, StopFrequency = 90e6, StepFrequency = 10e6, IFBandwidth = ifbw),
            VnaSegmentTableItem(StartFrequency = 100e6, StopFrequency = 70e9, StepFrequency = 100e6, IFBandwidth = ifbw)]),
        Z0 = 50,
        Sources = Array[VnaSourceSettings]([
            VnaSourceSettings(Power = power, Slope = 0),
            VnaSourceSettings(Power = power, Slope = 0)]),
        Ports = Array[VnaPortSettings]([
            VnaPortSettings(Attenuator = 0, Extension = 0),
            VnaPortSettings(Attenuator = 0, Extension = 0)]))          
    journal.AddItem(vnaitem)

def AddCustomConnectorSettingsToJournal(timestamp, journal):
    custom_conn = CustomConnectorSettingsJournalItem()
    custom_conn.TimeStamp = timestamp
    custom_conn.Connector = Connector(
        Identification = 'Custom Connector',
        Zr = 50.0,
        ElectricalSpecifications = Array[ConnectorElectricalSpec]([
            ConnectorElectricalSpec(Frequency = 0, Repeatability = -54),
            ConnectorElectricalSpec(Frequency = 70e9, Repeatability = -54)]))
    journal.AddItem(custom_conn)
    
def AddCustomCableSettingsToJournal(timestamp, journal):
    custom_cable = CustomCableSettingsJournalItem()
    custom_cable.TimeStamp = timestamp
    custom_cable.Cable = Cable(
        Identification = 'Custom Cable',
        Zr = 50.0,
        ElectricalSpecifications = Array[CableElectricalSpec]([
            CableElectricalSpec(Frequency = 0, StabilityMag = 0.02, StabilityPhase = 2),
            CableElectricalSpec(Frequency = 70e9, StabilityMag = 0.02, StabilityPhase = 2)]))
    journal.AddItem(custom_cable)
    
def CreateNewProject(timestamp):
    Directory.CreateDirectory(projectdir)
    Directory.CreateDirectory(measdir)
    journal = s.NewJournal()
    journal.RootPath = projectdir
    AddVnaSettingsToJournal(timestamp, journal)
    AddCustomCableSettingsToJournal(timestamp.AddSeconds(1), journal)
    AddCustomConnectorSettingsToJournal(timestamp.AddSeconds(2), journal)
    return journal

def AddMeasurementToJournal(timestamp, filename, journal):
    # copy data
    d = s.LoadSParamData(filename)
    if d is not None:
        print filename.replace(datadir + '\\', '')
        fi = FileInfo(filename.replace(datadir, measdir).replace('.s2p','.sdatb'))
        fi.Directory.Create()
        s.SaveSParamData(fi.FullName, d)
        File.SetLastWriteTime(fi.FullName, timestamp)
        # journal
        measitem = MeasurementJournalItem()
        measitem.TimeStamp = timestamp
        measitem.VnaDevice = vna
        measitem.MeasurementPath = fi.FullName.replace(projectdir + '\\', '')
        journal.AddNewDUTUncertaintyJournalItem(timestamp.AddSeconds(-4), dutunc)
        #journal.AddCableMovementJournalItem(timestamp.AddSeconds(-4), cable, 1)
        journal.AddCableMovementJournalItem(timestamp.AddSeconds(-3), cable, 2)
        journal.AddNewConnectionJournalItem(timestamp.AddSeconds(-2), connector, 1)
        journal.AddNewConnectionJournalItem(timestamp.AddSeconds(-1), connector, 2)
        journal.AddItem(measitem)
    else:
        print 'ERROR: ' + filename.replace(datadir + '\\', '')

# main
di = DirectoryInfo(datadir)
timestamps = List[DateTime]()
fis = di.GetFiles('*.*', SearchOption.AllDirectories)
for fi in fis:
    timestamps.Add(fi.LastWriteTime)
Array.Sort(timestamps.ToArray(), fis)
journal = CreateNewProject(fis[0].LastWriteTime.AddSeconds(-60))
for fi in fis:
    AddMeasurementToJournal(fi.LastWriteTime, fi.FullName, journal)
s.SaveJournal(journalfile, journal)
